@@ -109,7 +109,7 @@ process_header (CamelMedium *medium,
 	const gchar *charset;
 	gchar *unfolded;
 
-	header_type = (CamelHeaderType) g_hash_table_lookup (header_name_table, name);
+	header_type = (CamelHeaderType) GPOINTER_TO_INT (g_hash_table_lookup (header_name_table, name));
 	switch (header_type) {
 	case HEADER_FROM:
 		addr = camel_internet_address_new ();
@@ -1120,7 +1120,7 @@ best_encoding (CamelMimeMessage *msg,
  * Re-encode all message parts to conform with the required encoding rules.
  *
  * If @enctype is #CAMEL_BESTENC_7BIT, then all parts will be re-encoded into
- * one of the 7bit transfer encodings. If @enctype is #CAMEL_BESTENC_8bit, all
+ * one of the 7bit transfer encodings. If @enctype is #CAMEL_BESTENC_8BIT, all
  * parts will be re-encoded to either a 7bit encoding or, if the part is 8bit
  * text, allowed to stay 8bit. If @enctype is #CAMEL_BESTENC_BINARY, then binary
  * parts will be encoded as binary and 8bit textual parts will be encoded as 8bit.
@@ -1396,6 +1396,8 @@ cmm_dump_rec (CamelMimeMessage *msg,
 	gchar *s;
 	const GByteArray *data;
 
+	g_return_if_fail (CAMEL_IS_MIME_PART (part));
+
 	s = alloca (depth + 1);
 	memset (s, ' ', depth);
 	s[depth] = 0;
