@@ -73,7 +73,7 @@ test_create_object_sync (ETestServerFixture *fixture,
 	icalcomp = icalcomponent_new (ICAL_VEVENT_COMPONENT);
 	icalcomponent_set_summary (icalcomp, "Test event summary");
 	icalcomponent_set_dtstart (icalcomp, now);
-	icalcomponent_set_dtend   (icalcomp, icaltime_from_timet (icaltime_as_timet (now) + 60 * 60 * 60, 0));
+	icalcomponent_set_dtend   (icalcomp, icaltime_from_timet_with_zone (icaltime_as_timet (now) + 60 * 60 * 60, 0, NULL));
 
 	if (!e_cal_client_create_object_sync (cal_client, icalcomp, &uid, NULL, &error))
 		g_error ("create object sync: %s", error->message);
@@ -208,7 +208,7 @@ test_create_object_async (ETestServerFixture *fixture,
 	icalcomp = icalcomponent_new (ICAL_VEVENT_COMPONENT);
 	icalcomponent_set_summary (icalcomp, "Test event summary");
 	icalcomponent_set_dtstart (icalcomp, now);
-	icalcomponent_set_dtend   (icalcomp, icaltime_from_timet (icaltime_as_timet (now) + 60 * 60 * 60, 0));
+	icalcomponent_set_dtend   (icalcomp, icaltime_from_timet_with_zone (icaltime_as_timet (now) + 60 * 60 * 60, 0, NULL));
 
 	data.icalcomp = icalcomp;
 	data.loop = fixture->loop;
